@@ -1,6 +1,6 @@
 // Iostreams base classes -*- C++ -*-
 
-// Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002
+// Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003
 // Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
@@ -43,6 +43,8 @@
 #pragma GCC system_header
 
 #include <bits/atomicity.h>
+#include <bits/localefwd.h>
+#include <bits/locale_classes.h>
 
 namespace std
 {
@@ -145,11 +147,20 @@ namespace std
   enum _Ios_Seekdir { _M_ios_seekdir_end = 1L << 16 };
 
   // 27.4.2  Class ios_base
+  /**
+   *  @brief  The very top of the I/O class hierarchy.
+   *
+   *  This class defines everything that can be defined about I/O that does
+   *  not depend on the type of characters being input or output.  Most
+   *  people will only see @c ios_base when they need to specify the full
+   *  name of the various I/O flags (e.g., the openmodes).
+  */
   class ios_base
   {
   public:
     
     // 27.4.2.1.1  Class ios_base::failure
+    /// These are thrown to indicate problems.  Doc me.
     class failure : public exception
     {
     public:
@@ -173,56 +184,164 @@ namespace std
     };
 
     // 27.4.2.1.2  Type ios_base::fmtflags
+    /**
+     *  @brief This is a bitmask type.
+     *
+     *  @c "_Ios_Fmtflags" is implementation-defined, but it is valid to
+     *  perform bitwise operations on these values and expect the Right
+     *  Thing to happen.  Defined objects of type fmtflags are:
+     *  - boolalpha
+     *  - dec
+     *  - fixed
+     *  - hex
+     *  - internal
+     *  - left
+     *  - oct
+     *  - right
+     *  - scientific
+     *  - showbase
+     *  - showpoint
+     *  - showpos
+     *  - skipws
+     *  - unitbuf
+     *  - uppercase
+     *  - adjustfield
+     *  - basefield
+     *  - floatfield
+    */
     typedef _Ios_Fmtflags fmtflags;
-    // 27.4.2.1.2  Type fmtflags
+    /// Insert/extract @c bool in alphabetic rather than numeric format.
     static const fmtflags boolalpha =   fmtflags(__ios_flags::_S_boolalpha);
+    /// Converts integer input or generates integer output in decimal base.
     static const fmtflags dec =         fmtflags(__ios_flags::_S_dec);
+    /// Generate floating-point output in fixed-point notation.
     static const fmtflags fixed =       fmtflags(__ios_flags::_S_fixed);
+    /// Converts integer input or generates integer output in hexadecimal base.
     static const fmtflags hex =         fmtflags(__ios_flags::_S_hex);
+    /// Adds fill characters at a designated internal point in certain
+    /// generated output, or identical to @c right if no such point is
+    /// designated.
     static const fmtflags internal =    fmtflags(__ios_flags::_S_internal);
+    /// Adds fill characters on the right (final positions) of certain
+    /// generated output.  (I.e., the thing you print is flush left.)
     static const fmtflags left =        fmtflags(__ios_flags::_S_left);
+    /// Converts integer input or generates integer output in octal base.
     static const fmtflags oct =         fmtflags(__ios_flags::_S_oct);
+    /// Adds fill characters on the left (initial positions) of certain
+    /// generated output.  (I.e., the thing you print is flush right.)
     static const fmtflags right =       fmtflags(__ios_flags::_S_right);
+    /// Generates floating-point output in scientific notation.
     static const fmtflags scientific =  fmtflags(__ios_flags::_S_scientific);
+    /// Generates a prefix indicating the numeric base of generated integer
+    /// output.
     static const fmtflags showbase =    fmtflags(__ios_flags::_S_showbase);
+    /// Generates a decimal-point character unconditionally in generated
+    /// floating-point output.
     static const fmtflags showpoint =   fmtflags(__ios_flags::_S_showpoint);
+    /// Generates a + sign in non-negative generated numeric output.
     static const fmtflags showpos =     fmtflags(__ios_flags::_S_showpos);
+    /// Skips leading white space before certain input operations.
     static const fmtflags skipws =      fmtflags(__ios_flags::_S_skipws);
+    /// Flushes output after each output operation.
     static const fmtflags unitbuf =     fmtflags(__ios_flags::_S_unitbuf);
+    /// Replaces certain lowercase letters with their uppercase equivalents
+    /// in generated output.
     static const fmtflags uppercase =   fmtflags(__ios_flags::_S_uppercase);
+    /// A mask of left|right|internal.  Useful for the 2-arg form of @c setf.
     static const fmtflags adjustfield = fmtflags(__ios_flags::_S_adjustfield);
+    /// A mask of dec|oct|hex.  Useful for the 2-arg form of @c setf.
     static const fmtflags basefield =   fmtflags(__ios_flags::_S_basefield);
+    /// A mask of scientific|fixed.  Useful for the 2-arg form of @c setf.
     static const fmtflags floatfield =  fmtflags(__ios_flags::_S_floatfield);
 
     // 27.4.2.1.3  Type ios_base::iostate
+    /**
+     *  @brief This is a bitmask type.
+     *
+     *  @c "_Ios_Iostate" is implementation-defined, but it is valid to
+     *  perform bitwise operations on these values and expect the Right
+     *  Thing to happen.  Defined objects of type iostate are:
+     *  - badbit
+     *  - eofbit
+     *  - failbit
+     *  - goodbit
+    */
     typedef _Ios_Iostate iostate;
+    /// Indicates a loss of integrity in an input or output sequence (such
+    /// as an irrecoverable read error from a file).
     static const iostate badbit =  	iostate(__ios_flags::_S_badbit);
+    /// Indicates that an input operation reached the end of an input sequence.
     static const iostate eofbit =  	iostate(__ios_flags::_S_eofbit);
+    /// Indicates that an input operation failed to read the expected
+    /// characters, or that an output operation failed to generate the
+    /// desired characters.
     static const iostate failbit = 	iostate(__ios_flags::_S_failbit);
+    /// Indicates all is well.
     static const iostate goodbit = 	iostate(0);
 
-    // 27.4.2.1.4  Type openmode
+    // 27.4.2.1.4  Type ios_base::openmode
+    /**
+     *  @brief This is a bitmask type.
+     *
+     *  @c "_Ios_Openmode" is implementation-defined, but it is valid to
+     *  perform bitwise operations on these values and expect the Right
+     *  Thing to happen.  Defined objects of type openmode are:
+     *  - app
+     *  - ate
+     *  - binary
+     *  - in
+     *  - out
+     *  - trunc
+    */
     typedef _Ios_Openmode openmode;
+    /// Seek to end before each write.
     static const openmode app =    	openmode(__ios_flags::_S_app);
+    /// Open and seek to end immediately after opening.
     static const openmode ate =    	openmode(__ios_flags::_S_ate);
+    /// Perform input and output in binary mode (as opposed to text mode).
+    /// This is probably not what you think it is; see
+    /// http://gcc.gnu.org/onlinedocs/libstdc++/27_io/howto.html#3 and
+    /// http://gcc.gnu.org/onlinedocs/libstdc++/27_io/howto.html#7 for more.
     static const openmode binary = 	openmode(__ios_flags::_S_bin);
+    /// Open for input.  Default for @c ifstream and fstream.
     static const openmode in =     	openmode(__ios_flags::_S_in);
+    /// Open for output.  Default for @c ofstream and fstream.
     static const openmode out =    	openmode(__ios_flags::_S_out);
+    /// Open for input.  Default for @c ofstream.
     static const openmode trunc =  	openmode(__ios_flags::_S_trunc);
 
-    // 27.4.2.1.5  Type seekdir
+    // 27.4.2.1.5  Type ios_base::seekdir
+    /**
+     *  @brief This is an enumerated type.
+     *
+     *  @c "_Ios_Seekdir" is implementation-defined.  Defined values
+     *  of type seekdir are:
+     *  - beg
+     *  - cur, equivalent to @c SEEK_CUR in the C standard library.
+     *  - end, equivalent to @c SEEK_END in the C standard library.
+    */
     typedef _Ios_Seekdir seekdir;
+    /// Request a seek relative to the beginning of the stream.
     static const seekdir beg = 		seekdir(0);
+    /// Request a seek relative to the current position within the sequence.
     static const seekdir cur = 		seekdir(SEEK_CUR);
+    /// Request a seek relative to the current end of the sequence.
     static const seekdir end = 		seekdir(SEEK_END);
 
 #ifdef _GLIBCPP_DEPRECATED
+    // Annex D.6
     typedef int io_state;
     typedef int open_mode;
     typedef int seek_dir;
+    
+    typedef std::streampos streampos;
+    typedef std::streamoff streamoff;
 #endif
 
     // Callbacks;
+    /**
+     *  @doctodo
+    */
     enum event
     {
       erase_event,
@@ -230,18 +349,30 @@ namespace std
       copyfmt_event
     };
 
+    /**
+     *  @doctodo
+    */
     typedef void (*event_callback) (event, ios_base&, int);
 
+    /**
+     *  @doctodo
+    */
     void 
     register_callback(event_callback __fn, int __index);
 
   protected:
-    // Data Members
+    //@{
+    /**
+     *  @if maint
+     *  ios_base data members (doc me)
+     *  @endif
+    */
     streamsize 		_M_precision;
     streamsize 		_M_width;
     fmtflags 		_M_flags;
     iostate 		_M_exception;
     iostate 	       	_M_streambuf_state;
+    //@}
 
     // 27.4.2.6  Members for callbacks
     // 27.4.2.6  ios_base callbacks
@@ -285,6 +416,7 @@ namespace std
     _Words  		_M_word_zero;    
 
     // Guaranteed storage.
+    // The first 5 iword and pword slots are reserved for internal use.
     static const int 	_S_local_word_size = 8;
     _Words  		_M_local_word[_S_local_word_size];  
 
@@ -320,15 +452,32 @@ namespace std
       static void
       _S_ios_destroy();
 
+      // NB: Allows debugger applications use of the standard streams
+      // from operator new. _S_ios_base_init must be incremented in
+      // _S_ios_create _after_ initialization is completed.
+      static bool
+      _S_initialized() { return _S_ios_base_init; }
+
     private:
       static int 	_S_ios_base_init;
       static bool	_S_synced_with_stdio;
     };
 
-    // Fmtflags state:
+    // [27.4.2.2] fmtflags state functions
+    /**
+     *  @brief  Access to format flags.
+     *  @return  The format control flags for both input and output.
+    */
     inline fmtflags 
     flags() const { return _M_flags; }
 
+    /**
+     *  @brief  Setting new format flags all at once.
+     *  @param  fmtfl  The new flags to set.
+     *  @return  The previous format control flags.
+     *
+     *  This function overwrites all the format flags with @a fmtfl.
+    */
     inline fmtflags 
     flags(fmtflags __fmtfl)
     { 
@@ -337,6 +486,14 @@ namespace std
       return __old; 
     }
 
+    /**
+     *  @brief  Setting new format flags.
+     *  @param  fmtfl  Additional flags to set.
+     *  @return  The previous format control flags.
+     *
+     *  This function sets additional flags in format control.  Flags that
+     *  were previously set remain set.
+    */
     inline fmtflags 
     setf(fmtflags __fmtfl)
     { 
@@ -345,6 +502,15 @@ namespace std
       return __old; 
     }
 
+    /**
+     *  @brief  Setting new format flags.
+     *  @param  fmtfl  Additional flags to set.
+     *  @param  mask  The flags mask for @a fmtfl.
+     *  @return  The previous format control flags.
+     *
+     *  This function clears @a mask in the format flags, then sets
+     *  @a fmtfl @c & @a mask.  An example mask is @c ios_base::adjustfield.
+    */
     inline fmtflags 
     setf(fmtflags __fmtfl, fmtflags __mask)
     {
@@ -354,12 +520,32 @@ namespace std
       return __old;
     }
 
+    /**
+     *  @brief  Clearing format flags.
+     *  @param  mask  The flags to unset.
+     *
+     *  This function clears @a mask in the format flags.
+    */
     inline void 
     unsetf(fmtflags __mask) { _M_flags &= ~__mask; }
 
+    /**
+     *  @brief  Flags access.
+     *  @return  The precision to generate on certain output operations.
+     *
+     *  @if maint
+     *  Be careful if you try to give a definition of "precision" here; see
+     *  DR 189.
+     *  @endif
+    */
     inline streamsize 
     precision() const { return _M_precision; }
 
+    /**
+     *  @brief  Changing flags.
+     *  @param  prec  The new precision value.
+     *  @return  The previous value of precision().
+    */
     inline streamsize 
     precision(streamsize __prec)
     { 
@@ -368,9 +554,20 @@ namespace std
       return __old; 
     }
 
+    /**
+     *  @brief  Flags access.
+     *  @return  The minimum field width to generate on output operations.
+     *
+     *  "Minimum field width" refers to the number of characters.
+    */
     inline streamsize 
     width() const { return _M_width; }
 
+    /**
+     *  @brief  Changing flags.
+     *  @param  wide  The new width value.
+     *  @return  The previous value of width().
+    */
     inline streamsize 
     width(streamsize __wide)
     { 
@@ -379,20 +576,63 @@ namespace std
       return __old; 
     }
 
+    // [27.4.2.4] ios_base static members
+    /**
+     *  @brief  Interaction with the standard C I/O objects.
+     *  @param  sync  Whether to synchronize or not.
+     *  @return  True if the standard streams were previously synchronized.
+     *
+     *  The synchronization referred to is @e only that between the standard
+     *  C facilities (e.g., stdout) and the standard C++ objects (e.g.,
+     *  cout).  User-declared streams are unaffected.  See
+     *  http://gcc.gnu.org/onlinedocs/libstdc++/27_io/howto.html#8 for more.
+    */
     static bool 
     sync_with_stdio(bool __sync = true);
 
-    // Locales:
+    // [27.4.2.3] ios_base locale functions
+    /**
+     *  @brief  Setting a new locale.
+     *  @param  loc  The new locale.
+     *  @return  The previous locale.
+     *
+     *  Sets the new locale for this stream, and
+     *  [XXX does something with callbacks].
+    */
     locale 
     imbue(const locale& __loc);
 
+    /**
+     *  @brief  Locale access
+     *  @return  A copy of the current locale.
+     *
+     *  If @c imbue(loc) has previously been called, then this function
+     *  returns @c loc.  Otherwise, it returns a copy of @c std::locale(),
+     *  the global C++ locale.
+    */
     inline locale 
     getloc() const { return _M_ios_locale; }
 
-    // Storage:
+    /**
+     *  @brief  Locale access
+     *  @return  A reference to the current locale.
+     *
+     *  Like getloc above, but returns a reference instead of
+     *  generating a copy.
+    */
+    inline const locale& 
+    _M_getloc() const { return _M_ios_locale; }
+
+    // [27.4.2.5] ios_base storage functions
+    /**
+     *  @doctodo
+    */
     static int 
     xalloc() throw();
 
+    /**
+     *  @doctodo
+    */
     inline long& 
     iword(int __ix)
     {
@@ -401,6 +641,9 @@ namespace std
       return __word._M_iword;
     }
 
+    /**
+     *  @doctodo
+    */
     inline void*& 
     pword(int __ix)
     {
@@ -410,6 +653,10 @@ namespace std
     }
 
     // Destructor
+    /**
+     *  Destroys local storage and
+     *  [XXX does something with callbacks].
+    */
     ~ios_base();
 
   protected:
@@ -425,7 +672,8 @@ namespace std
 #endif
   };
  
-  // 27.4.5.1 fmtflags manipulators:
+  // [27.4.5.1] fmtflags manipulators
+  /// Calls base.setf(ios_base::boolalpha).
   inline ios_base& 
   boolalpha(ios_base& __base)
   {
@@ -433,6 +681,7 @@ namespace std
     return __base;
   }
 
+  /// Calls base.unsetf(ios_base::boolalpha).
   inline ios_base& 
   noboolalpha(ios_base& __base)
   {
@@ -440,6 +689,7 @@ namespace std
     return __base;
   }
 
+  /// Calls base.setf(ios_base::showbase).
   inline ios_base& 
   showbase(ios_base& __base)
   {
@@ -447,6 +697,7 @@ namespace std
     return __base;
   }
 
+  /// Calls base.unsetf(ios_base::showbase).
   inline ios_base& 
   noshowbase(ios_base& __base)
   {
@@ -454,6 +705,7 @@ namespace std
     return __base;
   }
 
+  /// Calls base.setf(ios_base::showpoint).
   inline ios_base& 
   showpoint(ios_base& __base)
   {
@@ -461,6 +713,7 @@ namespace std
     return __base;
   }
 
+  /// Calls base.unsetf(ios_base::showpoint).
   inline ios_base& 
   noshowpoint(ios_base& __base)
   {
@@ -468,6 +721,7 @@ namespace std
     return __base;
   }
 
+  /// Calls base.setf(ios_base::showpos).
   inline ios_base& 
   showpos(ios_base& __base)
   {
@@ -475,6 +729,7 @@ namespace std
     return __base;
   }
 
+  /// Calls base.unsetf(ios_base::showpos).
   inline ios_base& 
   noshowpos(ios_base& __base)
   {
@@ -482,6 +737,7 @@ namespace std
     return __base;
   }
 
+  /// Calls base.setf(ios_base::skipws).
   inline ios_base& 
   skipws(ios_base& __base)
   {
@@ -489,6 +745,7 @@ namespace std
     return __base;
   }
   
+  /// Calls base.unsetf(ios_base::skipws).
   inline ios_base& 
   noskipws(ios_base& __base)
   {
@@ -496,6 +753,7 @@ namespace std
     return __base;
   }
 
+  /// Calls base.setf(ios_base::uppercase).
   inline ios_base& 
   uppercase(ios_base& __base)
   {
@@ -503,6 +761,7 @@ namespace std
     return __base;
   }
 
+  /// Calls base.unsetf(ios_base::uppercase).
   inline ios_base& 
   nouppercase(ios_base& __base)
   {
@@ -510,6 +769,7 @@ namespace std
     return __base;
   }
 
+  /// Calls base.setf(ios_base::unitbuf).
   inline ios_base& 
   unitbuf(ios_base& __base)
   {
@@ -517,6 +777,7 @@ namespace std
      return __base;
   }
 
+  /// Calls base.unsetf(ios_base::unitbuf).
   inline ios_base& 
   nounitbuf(ios_base& __base)
   {
@@ -524,7 +785,8 @@ namespace std
      return __base;    
   }
 
-  // 27.4.5.2 adjustfield anipulators:
+  // [27.4.5.2] adjustfield anipulators
+  /// Calls base.setf(ios_base::internal, ios_base::adjustfield).
   inline ios_base& 
   internal(ios_base& __base)
   {
@@ -532,6 +794,7 @@ namespace std
      return __base;    
   }
 
+  /// Calls base.setf(ios_base::left, ios_base::adjustfield).
   inline ios_base& 
   left(ios_base& __base)
   {
@@ -539,6 +802,7 @@ namespace std
     return __base;
   }
   
+  /// Calls base.setf(ios_base::right, ios_base::adjustfield).
   inline ios_base& 
   right(ios_base& __base)
   {
@@ -546,7 +810,8 @@ namespace std
     return __base;
   }
   
-  // 27.4.5.3 basefield anipulators:
+  // [27.4.5.3] basefield anipulators
+  /// Calls base.setf(ios_base::dec, ios_base::basefield).
   inline ios_base& 
   dec(ios_base& __base)
   {
@@ -554,6 +819,7 @@ namespace std
     return __base;
   }
   
+  /// Calls base.setf(ios_base::hex, ios_base::basefield).
   inline ios_base& 
   hex(ios_base& __base)
   {
@@ -561,6 +827,7 @@ namespace std
     return __base;
   }
 
+  /// Calls base.setf(ios_base::oct, ios_base::basefield).
   inline ios_base& 
   oct(ios_base& __base)
   {
@@ -568,7 +835,8 @@ namespace std
     return __base;
   }
   
-  // 27.4.5.4 floatfield anipulators:
+  // [27.4.5.4] floatfield anipulators
+  /// Calls base.setf(ios_base::fixed, ios_base::floatfield).
   inline ios_base& 
   fixed(ios_base& __base)
   {
@@ -576,6 +844,7 @@ namespace std
     return __base;
   }
 
+  /// Calls base.setf(ios_base::scientific, ios_base::floatfield).
   inline ios_base& 
   scientific(ios_base& __base)
   {
